from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}
        user_paon = kwargs.get("paon")
        user_postcode = kwargs.get("postcode")
        check_paon(user_paon)
        check_postcode(user_postcode)

        # Create Selenium webdriver
        driver = create_webdriver()
        driver.get("https://www.gateshead.gov.uk/article/3150/Bin-collection-day-checker")

        # Wait for the postcode field to appear then populate it
        inputElement_postcode = WebDriverWait(driver, 30).until(
            EC.presence_of_element_located(
                (By.ID, "BINCOLLECTIONCHECKER_ADDRESSSEARCH_ADDRESSLOOKUPPOSTCODE"))
        )
        inputElement_postcode.send_keys(user_postcode)

        # Click search button
        findAddress = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located(
                (By.ID, "BINCOLLECTIONCHECKER_ADDRESSSEARCH_ADDRESSLOOKUPSEARCH"))
        )
        findAddress.click()

        # Wait for the 'Select address' dropdown to appear and select option matching the house name/number
        WebDriverWait(driver, 10).until(EC.element_to_be_clickable((
            By.XPATH,
            "//select[@id='BINCOLLECTIONCHECKER_ADDRESSSEARCH_ADDRESSLOOKUPADDRESS']//option[contains(., '" + user_paon + "')]"
        ))).click()

        # Wait for the collections table to appear
        WebDriverWait(driver, 10).until(EC.presence_of_element_located((By.CSS_SELECTOR, ".bincollections__table")))

        soup = BeautifulSoup(driver.page_source, features="html.parser")

        # Get collections table
        table = soup.find("table", {"class": "bincollections__table"})

        # Get rows
        month_year = ""
        for row in table.find_all("tr"):
            if row.find("th"):
                month_year = row.find("th").get_text(strip=True) + " " + datetime.now().strftime("%Y")
            elif month_year != "":
                collection = row.find_all("td")
                bin_date = datetime.strptime(collection[0].get_text(strip=True) + " " + month_year, "%d %B %Y")
                dict_data = {
                    "type": collection[2].get_text().replace("- DAY CHANGE", "").strip(),
                    "collectionDate": bin_date.strftime(date_format)
                }
                data["bins"].append(dict_data)

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return data



from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the base
    class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Create Selenium webdriver
        driver = create_webdriver()
        driver.get(kwargs.get("url"))

        # Make a BS4 object
        soup = BeautifulSoup(driver.page_source, features="html.parser")
        soup.prettify()

        data = {"bins": []}
        sections = soup.find_all("div", {"class": "wil_c-content-section_heading"})
        for s in sections:
            if s.get_text(strip=True).lower() == "bin collections":
                rows = s.find_next_sibling("div", {"class": "c-content-section_body"}).find_all(
                    "div", {"class": "u-mb-8"}
                )
                for row in rows:
                    title = row.find("div", {"class": "u-mb-4"})
                    collections = row.find_all("div", {"class": "u-mb-2"})
                    if title and collections:
                        for c in collections:
                            if c.get_text(strip=True).lower().startswith('next collection'):
                                # add next collection
                                next_collection_date = datetime.strptime(
                                    c.get_text(strip=True).replace('Next collection - ', ''),
                                    "%A, %d %B %Y",
                                ).strftime(date_format)
                                dict_data = {
                                    "type": title.get_text(strip=True).capitalize(),
                                    "collectionDate": next_collection_date,
                                }
                                data["bins"].append(dict_data)
                                # add future collections without duplicating next collection
                                future_collections = row.find("ul", {"class": "u-mt-4"}).find_all("li")
                                for c in future_collections:
                                    future_collection_date = datetime.strptime(
                                        c.get_text(strip=True),
                                        "%A, %d %B %Y",
                                    ).strftime(date_format)
                                    if future_collection_date != next_collection_date:
                                        dict_data = {
                                            "type": title.get_text(strip=True).capitalize(),
                                            "collectionDate": future_collection_date,
                                        }
                                        data["bins"].append(dict_data)

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        api_url = "https://forms.n-somerset.gov.uk/Waste/CollectionSchedule"
        uprn = kwargs.get("uprn")
        postcode = kwargs.get("postcode")
        check_uprn(uprn)
        check_postcode(postcode)

        # Get schedule from API
        values = {
            "PreviousHouse": "",
            "PreviousPostcode": postcode,
            "Postcode": postcode,
            "SelectedUprn": uprn,
        }
        headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64)"}
        requests.packages.urllib3.disable_warnings()
        response = requests.request("POST", api_url, headers=headers, data=values)

        soup = BeautifulSoup(response.text, features="html.parser")

        rows = soup.find("table", {"class": re.compile("table")}).find_all("tr")

        # Form a JSON wrapper
        data = {"bins": []}

        # Loops the Rows
        for row in rows:
            cells = row.find_all("td")
            if cells:
                binType = cells[0].get_text(strip=True)
                collectionDate = cells[1].get_text(strip=True) + " " + datetime.now().strftime("%Y")
                nextCollectionDate = cells[2].get_text(strip=True) + " " + datetime.now().strftime("%Y")

                # Make each Bin element in the JSON
                dict_data = {
                    "type": binType,
                    "collectionDate": get_next_occurrence_from_day_month(
                        datetime.strptime(collectionDate, "%A %d %B %Y")
                    ).strftime(date_format)
                }

                # Add data to the main JSON Wrapper
                data["bins"].append(dict_data)

                # Make each next Bin element in the JSON
                dict_data = {
                    "type": binType,
                    "collectionDate": get_next_occurrence_from_day_month(
                        datetime.strptime(nextCollectionDate, "%A %d %B %Y")
                    ).strftime(date_format),
                }

                # Add data to the main JSON Wrapper
                data["bins"].append(dict_data)

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return data



from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the base
    class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        # Pad UPRN with 0's at the start for any that aren't 12 chars
        user_uprn = user_uprn.zfill(12)

        # Create Selenium webdriver
        driver = create_webdriver()
        driver.get(
            f"https://my.reigate-banstead.gov.uk/en/service/Bins_and_recycling___collections_calendar?uprn={user_uprn}")

        # Wait for iframe to load and switch to it
        WebDriverWait(driver, 30).until(EC.frame_to_be_available_and_switch_to_it((By.ID, 'fillform-frame-1')))

        # Wait for form
        WebDriverWait(driver, 30).until(
            EC.presence_of_element_located((By.CSS_SELECTOR, 'span[data-name="html2"] > div')))

        # Make a BS4 object
        soup = BeautifulSoup(driver.page_source, features="html.parser")
        soup.prettify()

        data = {"bins": []}
        section = soup.find("span", {"data-name": "html2"})
        dates = section.find_all("div")
        for d in dates:
            date = d.find("h3")
            collections = d.find_all("li")
            if date and collections:
                collection_date = datetime.strptime(date.get_text(strip=True), "%A %d %B %Y").strftime(date_format)
                for c in collections:
                    collection_type = c.get_text(strip=True)
                    if c.get_text(strip=True):
                        dict_data = {
                            "type": collection_type,
                            "collectionDate": collection_date,
                        }
                        data["bins"].append(dict_data)

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass
import re

# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """


    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()
        
        regex_pattern = r'<li>([^<]+)<'
        
        data = {"bins": []}

        for bins in soup.select('div[class*="containererf"]'):
            
            collection_date = datetime.strptime(bins.find(id="content").text.strip(), "%A, %d/%m/%Y")
            bin_type_div = bins.find(id='main1')
            bin_types = bin_type_div.findAll('li')
            for bin_type in bin_types:

                bin_type_name = re.sub(r'\(.*\)', '', bin_type.text).strip()

                dict_data = {
                    "type": bin_type_name,
                    "collectionDate": collection_date.strftime(date_format)
                }
                data["bins"].append(dict_data)

        return data



from bs4 import BeautifulSoup
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        page = "https://selfservice.preston.gov.uk/service/Forms/FindMyNearest.aspx?Service=bins"

        data = {"bins": []}

        user_paon = kwargs.get("paon")
        user_postcode = kwargs.get("postcode")
        check_paon(user_paon)
        check_postcode(user_postcode)

        # Create Selenium webdriver
        driver = create_webdriver()
        driver.get(page)

        # If you bang in the house number (or property name) and postcode in the box it should find your property
        inputElement_address = driver.find_element(
            By.ID,
            "MainContent_txtAddress",
        )

        inputElement_address.send_keys(user_paon)
        inputElement_address.send_keys(" ")
        inputElement_address.send_keys(user_postcode)

        driver.find_element(
            By.ID,
            "btnSearch",
        ).click()

        # Wait for the 'Select your property' dropdown to appear and select the first result
        dropdown = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.ID, "MainContent_ddlSearchResults"))
        )
        # Create a 'Select' for it, then select the first address in the list
        # (Index 0 is "Make a selection from the list")
        dropdownSelect = Select(dropdown)
        dropdownSelect.select_by_index(1)

        # Wait for the 'View more' link to appear, then click it to get the full set of dates
        viewMoreLink = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.LINK_TEXT, "View more collection dates"))
        )
        viewMoreLink.click()

        soup = BeautifulSoup(driver.page_source, features="html.parser")

        topLevelSpan = soup.find(
            "span",
            id="lblCollectionDates"
        )

        collectionDivs = topLevelSpan.findChildren(recursive=False)
        for collectionDiv in collectionDivs:
            # Each date has two child divs - the date and the bin type
            # However both strings are in <b> tags so there are 4 children, the text is at:
            # Index 1 - date string i.e. 'Monday 01/01/2023'
            # Index 3 - bin type i.e. 'General waste'
            typeAndDateDivs = collectionDiv.findChildren()

            # Strip the day (i.e. Monday) out of the collection date string for parsing
            dateString = typeAndDateDivs[1].text.split(' ')[1]

            data["bins"].append(
                {
                    "type": typeAndDateDivs[3].text,
                    "collectionDate": datetime.strptime(dateString, "%d/%m/%Y").strftime(date_format)
                }
            )

        return data



from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def get_data(self, page) -> dict:
        # Make a BS4 object
        soup = BeautifulSoup(page, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        for month in soup.select('div[class*="bin-collection__month"]'):
            monthName = month.select('h3[class*="bin-collection__title"]')[
                0
            ].text.strip()
            for collectionDay in month.select('li[class*="bin-collection__item"]'):
                bin_type = collectionDay.select('span[class*="bin-collection__type"]')[
                    0
                ].text.strip()
                binCollection = (
                        collectionDay.select('span[class*="bin-collection__day"]')[
                            0
                        ].text.strip()
                        + ", "
                        + collectionDay.select('span[class*="bin-collection__number"]')[
                            0
                        ].text.strip()
                        + " "
                        + monthName
                )

                dict_data = {
                    "type": bin_type,
                    "collectionDate": datetime.strptime(binCollection, "%A, %d %B %Y").strftime(date_format),
                }

                data["bins"].append(dict_data)

        return data

    def parse_data(self, page: str, **kwargs) -> dict:
        page = "https://www.highpeak.gov.uk/findyourbinday"

        # Assign user info
        user_postcode = kwargs.get("postcode")
        user_paon = kwargs.get("paon")

        # Create Selenium webdriver
        driver = create_webdriver()
        driver.get(page)

        # Hide Cookies
        inputElement_hc = driver.find_element(
            By.CLASS_NAME, "cookiemessage__link--close"
        )
        inputElement_hc.click()

        # Enter postcode in text box and wait
        inputElement_pc = driver.find_element(
            By.ID, "FINDBINDAYSHIGHPEAK_POSTCODESELECT_POSTCODE"
        )
        inputElement_pc.send_keys(user_postcode)
        inputElement_pc.send_keys(Keys.ENTER)

        WebDriverWait(driver, 10).until(
            EC.presence_of_element_located(
                (By.ID, "FINDBINDAYSHIGHPEAK_ADDRESSSELECT_ADDRESS")
            )
        )

        # Select address from dropdown and wait
        inputElement_ad = Select(
            driver.find_element(By.ID, "FINDBINDAYSHIGHPEAK_ADDRESSSELECT_ADDRESS")
        )

        inputElement_ad.select_by_visible_text(user_paon)

        WebDriverWait(driver, 10).until(
            EC.presence_of_element_located(
                (By.ID, "FINDBINDAYSHIGHPEAK_ADDRESSSELECT_ADDRESSSELECTNEXTBTN_NEXT")
            )
        )

        # Submit address information and wait
        driver.find_element(
            By.ID, "FINDBINDAYSHIGHPEAK_ADDRESSSELECT_ADDRESSSELECTNEXTBTN_NEXT"
        ).click()

        WebDriverWait(driver, 10).until(
            EC.presence_of_element_located(
                (By.ID, "FINDBINDAYSHIGHPEAK_CALENDAR_MAINCALENDAR")
            )
        )

        # Read next collection information into Pandas
        table = driver.find_element(
            By.ID, "FINDBINDAYSHIGHPEAK_CALENDAR_MAINCALENDAR"
        ).get_attribute("outerHTML")

        # Parse data into dict
        data = self.get_data(table)

        return data



from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        page = "https://selfserve.derbyshiredales.gov.uk/renderform.aspx?t=103&k=9644C066D2168A4C21BCDA351DA2642526359DFF"

        data = {"bins": []}

        user_uprn = kwargs.get("uprn")
        user_postcode = kwargs.get("postcode")
        check_uprn(user_uprn)
        check_postcode(user_postcode)

        # Create Selenium webdriver
        driver = create_webdriver()
        driver.get(page)

        # Populate postcode field
        inputElement_postcode = driver.find_element(
            By.ID,
            "ctl00_ContentPlaceHolder1_FF2924TB",
        )
        inputElement_postcode.send_keys(user_postcode)

        # Click search button
        driver.find_element(
            By.ID,
            "ctl00_ContentPlaceHolder1_FF2924BTN",
        ).click()

        # Wait for the 'Select address' dropdown to appear and select option matching UPRN
        dropdown = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.ID, "ctl00_ContentPlaceHolder1_FF2924DDL"))
        )
        # Create a 'Select' for it, then select the matching URPN option
        dropdownSelect = Select(dropdown)
        dropdownSelect.select_by_value("U" + user_uprn)

        # Wait for the submit button to appear, then click it to get the collection dates
        submit = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.ID, "ctl00_ContentPlaceHolder1_btnSubmit"))
        )
        submit.click()

        soup = BeautifulSoup(driver.page_source, features="html.parser")

        bin_rows = soup.find("div", id="ctl00_ContentPlaceHolder1_pnlConfirmation") \
            .find("div", {"class": "row"}).find_all("div", {"class": "row"})
        if bin_rows:
            for bin_row in bin_rows:
                bin_data = bin_row.find_all("div")
                if bin_data and bin_data[0] and bin_data[1]:
                    collection_date = datetime.strptime(bin_data[0].get_text(strip=True), "%A%d %B, %Y")
                    dict_data = {
                        "type": bin_data[1].get_text(strip=True),
                        "collectionDate": collection_date.strftime(date_format),
                    }
                    data["bins"].append(dict_data)

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return data



import time
from datetime import timedelta

from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}
        collection_types = [
            "non recyclable waste",
            "food and garden",
            "paper and card",
            "glass, cans and plastics",
        ]

        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        week_details = soup.find("div", {"class": "bin-dir-snip"})
        week_dates = week_details.find("div", {"class": "clearfix"}).find("p")
        week_collections = week_details.find_all_next("h4")

        results = re.search(
            "([A-Za-z0-9 ]+) to ([A-Za-z0-9 ]+)", week_dates.get_text().strip()
        )
        if results:
            week_start = datetime.strptime(results.groups()[0], "%A %d %B %Y")
            week_end = datetime.strptime(results.groups()[1], "%A %d %B %Y")
            week_days = (
                week_start + timedelta(days=i)
                for i in range((week_end - week_start).days + 1)
            )

            week_collection_types = []
            for week_collection in week_collections:
                week_collection = (
                    week_collection.get_text().strip().lower().replace("-", " ")
                )
                for collection_type in collection_types:
                    if collection_type in week_collection:
                        week_collection_types.append(collection_type)

            collection_schedule = (
                soup.find("div", {"class": "serviceDetails"})
                .find("table")
                .find_all_next("tr")
            )

            for day in week_days:
                for row in collection_schedule:
                    schedule_type = row.find("th").get_text().strip()
                    results2 = re.search("([^(]+)", row.find("td").get_text().strip())
                    if results2:
                        schedule_day = results2[1].strip()
                        for collection_type in week_collection_types:
                            if collection_type in schedule_type.lower():
                                if (
                                    day.weekday()
                                    == time.strptime(schedule_day, "%A").tm_wday
                                ):
                                    dict_data = {
                                        "type": schedule_type,
                                        "collectionDate": day.strftime(date_format),
                                    }
                                    data["bins"].append(dict_data)

        return data



import time
from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}
        user_uprn = kwargs.get("uprn")
        user_postcode = kwargs.get("postcode")
        check_uprn(user_uprn)
        check_postcode(user_postcode)

        # Create Selenium webdriver
        driver = create_webdriver()
        driver.get("https://www.npt.gov.uk/2195")

        # Accept cookies banner
        cookieAccept = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located(
                (By.ID, "ccc-notify-accept"))
        )
        cookieAccept.click()

        # Populate postcode field
        inputElement_postcode = driver.find_element(
            By.ID,
            "ContentPlaceHolderDefault_ctl13_nptLLPG2_25_addresslookup_txtTmpPostcode",
        )
        inputElement_postcode.send_keys(user_postcode)

        # Click search button
        findAddress = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located(
                (By.ID, "ContentPlaceHolderDefault_ctl13_nptLLPG2_25_addresslookup_btnFindAddress"))
        )
        findAddress.click()

        time.sleep(1)

        # Wait for the 'Select address' dropdown to appear and select option matching UPRN
        dropdown = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located(
                (By.ID, "ContentPlaceHolderDefault_ctl13_nptLLPG2_25_addresslookup_ddlAddressLookup"))
        )
        # Create a 'Select' for it, then select the matching URPN option
        dropdownSelect = Select(dropdown)
        dropdownSelect.select_by_value(user_uprn)

        # Remove back to top button if exists
        driver.execute_script("""
        if (document.contains(document.querySelector(".backtotop"))) {
            document.querySelector(".backtotop").remove();
        }
        """)

        # Wait for the submit button to appear, then click it to get the collection dates
        submit = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.ID, "ContentPlaceHolderDefault_ctl13_nptLLPG2_25_btnDisplay"))
        )
        submit.click()

        soup = BeautifulSoup(driver.page_source, features="html.parser")

        # Get the property details
        property_details = soup.find(
            "div",
            {"id": "ContentPlaceHolderDefault_ctl13_nptLLPG2_25_divPropertyDetails"}
        )

        # Get the dates
        for date in property_details.find_all("h2"):
            if date.get_text(strip=True) != "Bank Holidays":
                bin_date = datetime.strptime(
                    date.get_text(strip=True).replace('&nbsp', ' ') + " " + datetime.now().strftime("%Y"),
                    "%A, %d %B %Y"
                )
                bin_types_wrapper = date.find_next_sibling("div")
                for bin_type_wrapper in bin_types_wrapper.find_all("div", {"class": "card"}):
                    if bin_date and bin_type_wrapper:
                        bin_type = bin_type_wrapper.find("a").get_text(strip=True)
                        bin_type += " (" + bin_type_wrapper.find("span").get_text(strip=True) + ")"
                        dict_data = {
                            "type": bin_type,
                            "collectionDate": bin_date.strftime(date_format)
                        }
                        data["bins"].append(dict_data)

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return data



from bs4 import BeautifulSoup
from datetime import datetime
import requests
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass

COLLECTION_KINDS = {
    "waste": "rteelem_ctl03_pnlCollections_Refuse",
    "recycling": "rteelem_ctl03_pnlCollections_Recycling",
    "glass": "rteelem_ctl03_pnlCollections_Glass",
    # Garden waste data is only returned if the property is subscribed to the Garden Waste service
    "garden": "rteelem_ctl03_pnlCollections_GardenWaste"
}


class CouncilClass(AbstractGetBinDataClass):
    def parse_data(self, page: str, **kwargs) -> dict:
        requests.packages.urllib3.disable_warnings()

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        cookies = {
            'cookie_control_popup': 'A',
            'WhenAreMyBinsCollected': f'{user_uprn}',
        }

        headers = {
            'Accept': '*/*',
            'Accept-Language': 'en-GB,en;q=0.9',
            'Cache-Control': 'no-cache',
            'Connection': 'keep-alive',
            'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
            'Origin': 'https://www.basingstoke.gov.uk',
            'Pragma': 'no-cache',
            'Referer': 'https://www.basingstoke.gov.uk/rte.aspx?id=1270',
            'Sec-Fetch-Dest': 'empty',
            'Sec-Fetch-Mode': 'cors',
            'Sec-Fetch-Site': 'same-origin',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/116.0.5845.188 Safari/537.36',
            'X-MicrosoftAjax': 'Delta=true',
            'X-Requested-With': 'XMLHttpRequest',
        }

        params = {
            'id': '1270',
        }

        data = f'rteelem%24ctl03%24ctl00=rteelem%24ctl03%24ctl01%7Crteelem%24ctl03%24gapAddress%24ctl05&rteelem%24ctl03%24gapAddress%24lstStage2_SearchResults=UPRN%3A{user_uprn}&__EVENTTARGET=rteelem%24ctl03%24gapAddress%24ctl05&__EVENTARGUMENT=&__VIEWSTATE=%2FwEPDwUKLTQ2NzE5Mjc0NQ9kFgJmD2QWAgICD2QWAgIBD2QWAgIHD2QWAmYPZBYEAgEPDxYEHhNBc3NvY2lhdGVkQ29udHJvbElEBSJnYXBBZGRyZXNzOmxzdFN0YWdlMl9TZWFyY2hSZXN1bHRzHgRUZXh0BQ5TZWxlY3QgYWRkcmVzc2RkAgMPDxYEHiBHYXBFeHRlcm5hbFByb21wdExhYmVsVGV4dFN0YWdlMQUSU2VhcmNoIGZvciBhZGRyZXNzHiBHYXBFeHRlcm5hbFByb21wdExhYmVsVGV4dFN0YWdlMgUOU2VsZWN0IGFkZHJlc3NkFgJmD2QWBGYPDxYCHgdWaXNpYmxlaGQWAmYPDxYCHwEFCFJHMjIgNlRIZGQCAQ8PFgIfBGdkFgJmDxBkEBUMNDEzOCBTdCBQZXRlcnMgUm9hZCwgQmFzaW5nc3Rva2UsIEhhbXBzaGlyZSwgUkcyMiA2VEg0MTQwIFN0IFBldGVycyBSb2FkLCBCYXNpbmdzdG9rZSwgSGFtcHNoaXJlLCBSRzIyIDZUSDQxNDIgU3QgUGV0ZXJzIFJvYWQsIEJhc2luZ3N0b2tlLCBIYW1wc2hpcmUsIFJHMjIgNlRINDE0NCBTdCBQZXRlcnMgUm9hZCwgQmFzaW5nc3Rva2UsIEhhbXBzaGlyZSwgUkcyMiA2VEg0MTQ2IFN0IFBldGVycyBSb2FkLCBCYXNpbmdzdG9rZSwgSGFtcHNoaXJlLCBSRzIyIDZUSDQxNDggU3QgUGV0ZXJzIFJvYWQsIEJhc2luZ3N0b2tlLCBIYW1wc2hpcmUsIFJHMjIgNlRINDE1MCBTdCBQZXRlcnMgUm9hZCwgQmFzaW5nc3Rva2UsIEhhbXBzaGlyZSwgUkcyMiA2VEg0MTUyIFN0IFBldGVycyBSb2FkLCBCYXNpbmdzdG9rZSwgSGFtcHNoaXJlLCBSRzIyIDZUSDQxNTQgU3QgUGV0ZXJzIFJvYWQsIEJhc2luZ3N0b2tlLCBIYW1wc2hpcmUsIFJHMjIgNlRINDE1NiBTdCBQZXRlcnMgUm9hZCwgQmFzaW5nc3Rva2UsIEhhbXBzaGlyZSwgUkcyMiA2VEg0MTU4IFN0IFBldGVycyBSb2FkLCBCYXNpbmdzdG9rZSwgSGFtcHNoaXJlLCBSRzIyIDZUSDQxNjAgU3QgUGV0ZXJzIFJvYWQsIEJhc2luZ3N0b2tlLCBIYW1wc2hpcmUsIFJHMjIgNlRIFQwRVVBSTjoxMDAwNjAyNDM5MjcRVVBSTjoxMDAwNjAyNDM5MjkRVVBSTjoxMDAwNjAyNDM5MzERVVBSTjoxMDAwNjAyNDM5MzMRVVBSTjoxMDAwNjAyNDM5MzURVVBSTjoxMDAwNjAyNDM5MzYRVVBSTjoxMDAwNjAyNDM5MzcRVVBSTjoxMDAwNjAyNDM5MzgRVVBSTjoxMDAwNjAyNDM5MzkRVVBSTjoxMDAwNjAyNDM5NDARVVBSTjoxMDAwNjAyNDM5NDERVVBSTjoxMDAwNjAyNDM5NDIUKwMMZ2dnZ2dnZ2dnZ2dnZGRkpXCIF40J9nPqukmdVM4NgNZFZyw%3D&__VIEWSTATEGENERATOR=99691FF6&__EVENTVALIDATION=%2FwEdABCb2eofM0yrOZt2P3lnE8LBzdIwLRuYuP7lVS1GO2hXAAf%2FiyMIUYr%2BX38W%2FCsEufkYF%2FJqBocIUvPBZShq0SWLlDuEZpde9d1EPv1cdNAxtv0a5P%2BAzvWcKULA75C%2FHDNl8al%2FKtVDH8iZIW8%2BPWamtUNjyfZaTGu1VxFRW7%2BrIZHFk8PySEuoYzdlb%2Fw0NMLP8MZHHy%2BSyI7El1raMGfVGyh7Lv3Ohzid1s46Z3mtovjgyLnG9kXo%2FMyI4mgBTTdOYHrncJX8sN52g9M2NHMrNJrGEa%2BGwkZVSfqAxtisKhbq%2Bzxiu%2BV7mP9nRlRrnJ0yunAhZS1%2FkWU9mq7vbq4HclDPJK5tGeZ7jNpUx3wTgU%2Btyxc%3D&__ASYNCPOST=true&'

        response = requests.post('https://www.basingstoke.gov.uk/rte.aspx', params=params, cookies=cookies,
                                 headers=headers, data=data, verify=False)

        if response.status_code != 200:
            raise SystemError("Error retrieving data! Please try again or raise an issue on GitHub!")

        # Make a BS4 object
        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        bins = []

        for collection_type, collection_class in COLLECTION_KINDS.items():
            for date in soup.select(f"div#{collection_class} li"):
                bins.append({
                    "type": collection_type,
                    "collectionDate": datetime.strptime(
                        # Friday, 21 July 2023
                        date.get_text(strip=True),
                        '%A, %d %B %Y'
                    ).strftime(date_format)
                })

        return {
            "bins": bins
        }



from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        page = "https://selfservice.broxtowe.gov.uk/renderform.aspx?t=217&k=9D2EF214E144EE796430597FB475C3892C43C528"

        data = {"bins": []}

        user_uprn = kwargs.get("uprn")
        user_postcode = kwargs.get("postcode")
        check_uprn(user_uprn)
        check_postcode(user_postcode)

        # Create Selenium webdriver
        driver = create_webdriver()
        driver.get(page)

        # Populate postcode field
        inputElement_postcode = driver.find_element(
            By.ID,
            "ctl00_ContentPlaceHolder1_FF5683TB",
        )
        inputElement_postcode.send_keys(user_postcode)

        # Click search button
        driver.find_element(
            By.ID,
            "ctl00_ContentPlaceHolder1_FF5683BTN",
        ).click()

        # Wait for the 'Select address' dropdown to appear and select option matching UPRN
        dropdown = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.ID, "ctl00_ContentPlaceHolder1_FF5683DDL"))
        )
        # Create a 'Select' for it, then select the matching URPN option
        dropdownSelect = Select(dropdown)
        dropdownSelect.select_by_value("U" + user_uprn)

        # Wait for the submit button to appear, then click it to get the collection dates
        submit = WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.ID, "ctl00_ContentPlaceHolder1_btnSubmit"))
        )
        submit.click()

        soup = BeautifulSoup(driver.page_source, features="html.parser")

        bins_div = soup.find("div", id="ctl00_ContentPlaceHolder1_FF5686FormGroup")
        if bins_div:
            bins_table = bins_div.find("table")
            if bins_table:
                # Get table rows
                for row in bins_table.find_all("tr"):
                    # Get the rows cells
                    cells = row.find_all("td")
                    bin_type = cells[0].get_text(strip=True)
                    # Skip header row
                    if bin_type and cells[3] and bin_type != "Bin Type":
                        collection_date = datetime.strptime(cells[3].get_text(strip=True), "%A, %d %B %Y")
                        dict_data = {
                            "type": bin_type,
                            "collectionDate": collection_date.strftime(date_format),
                        }
                        data["bins"].append(dict_data)

                        data["bins"].sort(
                            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
                        )

        return data



import json

import requests
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # User email from @Home app as UPRN
        user_email = kwargs.get("uprn")
        headers = {
            "X-country": "gb",
            "X-email": user_email,
            "Connection": "Keep-Alive",
        }

        # Sniffed from the app
        requests.packages.urllib3.disable_warnings()
        response = requests.get(
            "https://services.athomeapp.net/ServiceData/GetUserRoundJson",
            headers=headers,
        )

        # 200 is OK. Sometimes it times out and gives this, but I'm not parsing HTTP codes
        if response.status_code != 200:
            raise ValueError(
                "Error parsing API. Please check your email is correct and registered on the @Home app."
            )

        # Load in the json and only get the bins
        json_data = json.loads(response.text)["userrounds"]
        data = {"bins": []}
        collections = []

        # For each bin, run through the list of dates and add them to a collection
        for item in json_data:
            bin_type = item["containername"]
            for sched in item["nextcollectiondates"]:
                bin_collection = datetime.strptime(
                    sched["datestring"], "%d %m %Y %H:%M"
                )
                if bin_collection.date() >= datetime.now().date():
                    collections.append((bin_type, bin_collection))

        # Order the collection of bins and dates by date order, then add to dict
        ordered_data = sorted(collections, key=lambda x: x[1])
        data = {"bins": []}
        for item in ordered_data:
            dict_data = {
                "type": item[0],
                "collectionDate": item[1].strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data



from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_uprn = kwargs.get("uprn")
        user_postcode = kwargs.get("postcode")
        check_uprn(user_uprn)
        check_postcode(user_postcode)

        # Make SOAP Request
        response = requests.post(
            "https://ccdc.opendata.onl/DynamicCall.dll",
            data="Method=CollectionDates&Postcode=" + user_postcode + "&UPRN=" + user_uprn,
            headers={
                "Content-Type": "application/x-www-form-urlencoded",
                "Referer": "https://ccdc.opendata.onl/CCDC_WasteCollection",
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/118.0.0.0 Safari/537.36"
            }
        )

        # Make a BS4 object
        soup = BeautifulSoup(response.text, "xml")
        soup.prettify()

        if soup.find("ErrorDescription") and soup.find("ErrorDescription").get_text(
                strip=True) == "No results returned":
            raise ValueError("No collection data found for provided Postcode & UPRN.")

        data = {"bins": []}

        collections = soup.find_all("Collection")

        for i in range(len(collections)):
            dict_data = {
                "type": collections[i].Service.get_text().replace("Collection Service", "").strip(),
                "collectionDate": datetime.strptime(
                    collections[i].Date.get_text(),
                    "%d/%m/%Y %H:%M:%S"
                ).strftime(date_format)
            }
            data["bins"].append(dict_data)

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )
        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        collections = []
        for month in soup.find_all("div", {"class": "calendar__print"}):
            current_month_name = month.find_next("h2").text
            general_bins = month.find_all("li", {"class": "active-refuse day"})
            recycling_bins = month.find_all("li", {"class": "active-recycling day"})
            bin_list = general_bins + recycling_bins
            for bin in bin_list:
                if bin.attrs.get("class")[0] == "active-recycling":
                    bin_type = "Recycling/Glass"
                elif bin.attrs.get("class")[0] == "active-refuse":
                    bin_type = "General waste/Garden"
                bin_date = datetime.strptime(bin.text.strip().capitalize() + " " + current_month_name + " " +
                                             str(datetime.now().year), '%A %d %B %Y')
                collections.append((bin_type, bin_date))

            # It'd be really hard to deal with next year, so just get December then end
            if current_month_name == "December":
                break

        ordered_data = sorted(collections, key=lambda x: x[1])
        data = {"bins": []}
        for item in ordered_data:
            dict_data = {
                "type": item[0],
                "collectionDate": item[1].strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data
